{% load url from future %}
<!DOCTYPE html>
<html>
    <head>
        <meta charset="utf-8" />
        <title>Redis_Cache db-status</title>
        <script type="text/javascript" src="{{ STATIC_URL }}redis_cache/jquery-1.6.2.min.js"></script>
        <script type="text/javascript" src="{{ STATIC_URL }}redis_cache/script.js"></script>
        <link type="text/css" media="screen" rel="stylesheet" href="{{ STATIC_URL }}redis_cache/style.css" />
    </head>
    <body>
        <div class="stats-header"></div>
        <div class="stats-body" rel="{% url 'redis_cache:redis_cache_status' %}"></div>
    </body>
    <!-- Copyright (c) 2011 Andrei Antoukh <niwi@niwi.be> -->
</html>
