body {
    font: normal 13px arial,sans-serif;
}

.stats-body table {
    width: 900px;
}
.stats-body table th {
    text-align: left;
}
.stats-body table td.dbname {
    vertical-align: top;
    color: #666;
    font-style: italic;
    width: 120px;
    padding: 5px;
}
.stats-body table td.dbname .djangoname {
    font-weight: bold;
}
.stats-body table td.dbname .smalldetails {
    font-size: 11px;
    font-style: italic;
    color: #333;
}
.stats-body table td.dbextra ul,
.stats-body table td.dbinfo ul {
    margin: 0px;
    padding: 0px;
    padding-bottom: 6px;
    list-style: none;
}
.stats-body table td.dbinfo,
.stats-body table td.dbextra {
    vertical-align: top;
}
